/*
 *	Copyright (C) 2007, 2008, 2009. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qv3dcanvas.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <QMouseEvent>

#include <qvipp.h>
#include <qvdefines.h>
#include <qvmatrixalgebra.h>

#include <qvgui/qv3dcanvas.h>
#include <QTimer>
#define GL_VIEW_ASPECT   1.333


QV3DCanvas::QV3DCanvas(	const QString &title, const double zoom,
			bool dr_center, const QColor &backgroundColor, QWidget* parent): QGLWidget(parent),
			trackballQuat(QVQuaternion::trackball(0.0, 0.0, 0.0, 0.0)), cx(0), cy(0), cz(0),
			dr_center(dr_center), zoom(zoom), fov(45), pressedleft(FALSE), pressedright(FALSE), colorCursor(0), backgroundColor(backgroundColor)
	{
	setName(title);

	resize(400,(int)(400/GL_VIEW_ASPECT));
	setWindowTitle(title);

	QTimer *timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(updateGL()));
	timer->start(1000/24);

	show();
	}

QV3DCanvas::~QV3DCanvas ()
	{
	// FIXME: is it needed?: makeCurrent(); 
	}

void QV3DCanvas::initializeGL()
	{
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	glDisable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);
	
	glEnable(GL_DITHER);
	glShadeModel(GL_SMOOTH);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
	glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
	
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST); 
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST); 
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);

	init();
	}

void QV3DCanvas::draw_center_of_rotation()
	{
	/* Yellow cross: (center of rotation). */
	glBegin(GL_LINES);
		glColor3ub(255,255,0);
		
		glVertex3f(cx - 0.1,cy,cz);
		glVertex3f(cx + 0.1,cy,cz);
		
		glVertex3f(cx,cy- 0.1,cz);
		glVertex3f(cx,cy+ 0.1,cz);
		
		glVertex3f(cx,cy,cz- 0.1);
		glVertex3f(cx,cy,cz+ 0.1);
	glEnd();
	}

#define S	15
void QV3DCanvas::paintGL()
	{
	glMatrixMode(GL_PROJECTION);

	glLoadIdentity();
	gluPerspective(fov,(float)size().width()/(float)size().height(),1,100*zoom);
	glMatrixMode(GL_MODELVIEW);

	qglClearColor(backgroundColor);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	
	glTranslatef(0,0,-4*zoom); 
	glScaled((GLdouble)0.5/zoom,(GLdouble)0.5/zoom,(GLdouble)0.5/zoom);

	QVMatrix M(trackballQuat);
	GLfloat m[4][4];

	for (int i=0; i <4; i++)
		for (int j=0; j <4; j++)
			m[i][j] = M(i,j);

	glMultMatrixf(&m[0][0]);
	
	glRotatef(270,1,0,0); 
	
	glTranslatef(-cx,-cy,-cz); 

	if(dr_center)
		draw_center_of_rotation();

	glScaled((GLdouble)0.5,(GLdouble)0.5,(GLdouble)0.5);

	viewer();

	display();
	}

void QV3DCanvas::resizeGL( int w, int h )
	{
	glViewport(0,0,w,h); 
	reshape(w,h);

	glScaled((GLdouble)0.5,(GLdouble)0.5,(GLdouble)0.5);

	viewer();
	}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////   Mouse & keyboard:     ///////////////////////
////////////////////////////////////////////////////////////////////////////////

void QV3DCanvas::mousePressEvent(QMouseEvent *event)
{
    beginx = event->x();
    beginy = event->y();
    if(event->button() == Qt::LeftButton) {
        pressedleft = TRUE;
    } else if(event->button() == Qt::RightButton) {
        pressedright = TRUE;
    }
}

void QV3DCanvas::mouseReleaseEvent(QMouseEvent *event)
{
    if(event->button() == Qt::LeftButton) {
        pressedleft = FALSE;
    } else if(event->button() == Qt::RightButton) {
        pressedright = FALSE;
    }
}

void QV3DCanvas::mouseMoveEvent(QMouseEvent *event)
{
    int x,y;

    x = (int) event->x();
    y = (int) event->y();

    if (pressedleft) {
	QVQuaternion spinQuat = QVQuaternion::trackball(
			(2.0*beginx - size().width())	/ size().width(),
			(size().height() - 2.0*beginy)	/ size().height(),
			(2.0*x - size().width())	/ size().width(),
			(size().height() - 2.0*y)	/ size().height());

	trackballQuat = spinQuat * trackballQuat;
        updateGL();
    }
/*
    if (pressedright) {
        //printf("MouseMove right button\n");
        zoom += ((y - beginy) / size().height()) * 100;

        //if (zoom < 5) zoom = 5;
        //if (zoom > 120) zoom = 120;
        updateGL();
    }*/
    beginx = x;
    beginy = y;
}

void QV3DCanvas::wheelEvent(QWheelEvent *event)
	{
	zoom *= pow(0.9995,event->delta());
	updateGL();
	}

void QV3DCanvas::keyPressEvent(QKeyEvent *event)
{
    //printf("KeyPress\n");
    switch(event->key()) {
      case Qt::Key_Left:
        cx -= 0.1; // maxwide/20;
        break;
      case Qt::Key_Right:
        cx += 0.1; // maxwide/20;
        break;
      case Qt::Key_Up:
        cy += 0.1; // maxwide/20;
        break;
      case Qt::Key_Down:
        cy -= 0.1; // maxwide/20;
        break;
      case Qt::Key_PageUp:
        cz += 0.1; // maxwide/20;
        break;
      case Qt::Key_PageDown:
        cz -= 0.1; // maxwide/20;
        break;
      case 'C':
        dr_center = not dr_center;
        break;
      //case 'B':
        //dr_box = not dr_box;
        //break;
    }
    /* Something changed; redraw: */
    updateGL();
}

void QV3DCanvas::closeEvent(QCloseEvent * event)
{
	Q_UNUSED(event);
	emit closed();
}

//////////////////////////////////////////////
#include <iostream>
void QV3DCanvas::viewer()
	{
	qDebug() << "QVImageCanvas::viewer()";
	readInputProperties();

	// Draw other objects
	foreach(QString name, getPropertyList())
		if (getPropertyFlags(name) & inputFlag)
			{
			// Draw QV3DPolylineF
			if (isType< QV3DPolylineF >(name))
				draw(	getPropertyValue< QV3DPolylineF >(name),
					getPropertyValue<QColor>("Color for " + name),
					getPropertyValue<double>("Size for " + name));
			// Draw QList<QV3DPointF>
			else if (isType< QList<QV3DPointF> >(name))
				draw(	getPropertyValue< QList<QV3DPointF> >(name),
					getPropertyValue<QColor>("Color for " + name),
					getPropertyValue<double>("Size for " + name));
			// Draw QList<QV3DPolylineF>
			else if (isType< QList<QV3DPolylineF> >(name))
				foreach(QV3DPolylineF polyline, getPropertyValue< QList<QV3DPolylineF> >(name))
					draw(	polyline,
						getPropertyValue<QColor>("Color for " + name));
			}

	foreach(QV3DModel *model, models)
		model->updatePaint(*this);

	glFlush();
	qDebug() << "QVImageCanvas::viewer() -> return";
	}

void QV3DCanvas::draw(const QV3DPolylineF &qv3DPolyline, const QColor color, const double size)
	{
	glPointSize(size);
	glBegin(GL_LINES);
	qglColor(color);
	for (int i = 1; i < qv3DPolyline.size(); i++)
		{
		glVertex3f(qv3DPolyline[i].x(), qv3DPolyline[i].y(), qv3DPolyline[i].z());
		glVertex3f(qv3DPolyline[i-1].x(), qv3DPolyline[i-1].y(), qv3DPolyline[i-1].z());
		}
	glEnd();
	}

#include <GL/glu.h>
void QV3DCanvas::draw(const QList<QV3DPointF> &qv3DPointList, const QColor color, const double size)
	{
	glPointSize(size);
	glEnable(GL_POINT_SMOOTH);
	glBegin( GL_POINTS );
	qglColor(color);
	foreach(QV3DPointF point, qv3DPointList)
		glVertex3f(point.x(), point.y(), point.z());
	glEnd();

	glBegin(GL_LINES);
	glColor3ub(color.red()/2, color.green()/2, color.blue()/2);
	foreach(QV3DPointF point, qv3DPointList)
		{
		glVertex3f(point.x(), point.y(), point.z());
		glVertex3f(point.x(), point.y(), 0);
		}
	glEnd();

	}

bool QV3DCanvas::linkUnspecifiedInputProperty(QVPropertyContainer *sourceContainer, QString sourcePropName, LinkType linkType)
	{
	if (linkType == SynchronousLink)
		{
		std::cerr	<< "ERROR: QVImageCanvas::linkUnspecifiedInputProperty():"
				<< " the linkType must be AsynchronousLink, the link will not be done"
				<< std::endl;
		return false;
		}

	QVWorker* worker;
	if((worker = dynamic_cast<QVWorker*>(sourceContainer)) != NULL)
		{
		if (	worker->isType< QV3DPolylineF >(sourcePropName)		||
			worker->isType< QList<QV3DPointF> >(sourcePropName)	||
			worker->isType< QList<QV3DPolylineF> >(sourcePropName)		)
			{
			addPropertyFromQVariant(sourcePropName, inputFlag, worker->getPropertyQVariantValue(sourcePropName), worker->getPropertyInfo(sourcePropName));
			addProperty<QColor>("Color for " + sourcePropName, inputFlag, getNextColor(), QString("Color for object ") + sourcePropName);
			addProperty<double>("Size for " + sourcePropName, inputFlag, 1, QString("Size for object ") + sourcePropName);
			}
		bool result = worker->linkProperty(sourcePropName, this, sourcePropName ,QVWorker::AsynchronousLink);
		//QObject::connect(worker, SIGNAL(endIteration(uint, int)), this, SLOT(refreshImageArea()));
		return result;
		}
	else
		{
		std::cerr << "ERROR: QVImageCanvas::linkUnspecifiedInputProperty(): the sourceContainer container must be a QVWorker, the link will not be done" << std::endl;
		return false;
		}

	}
