/*
 *	Copyright (C) 2007, 2008, 2009. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvnumericplot.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef NUMERICPLOT_H
#define NUMERICPLOT_H

#include <qvgui/qvplot.h>

/*!

@class QVNumericPlot qvgui/qvnumericplot.h
@brief Class for plot graphs of worker's int or double properties.

This class lets us plot graphs for int or double worker's output properties over a timeline.

The following code features how to create a Numeric plot object in a QVision application:

\code
void main()
	{
	[...]
	QVGraphsWorker graphsWorker("Graphs worker");
	[...]
	QVNumericPlot numericPlot("Numeric Plot", true, 10);
	graphsWorker.linkProperty("Max pixel", numericPlot);
	graphsWorker.linkProperty("Min pixel", numericPlot);
	[...]
	}
\endcode

This application will display a plot like the following:

\image html qvision_numericplot_window.png

By default, the @ref QVDefaultGUI includes the following button for CPU plot statistics displaying, which opens a @ref QVNumericPlot widget plotting the worker CPU usage:

\image html qvision_gui_graphbutton.png

For further info see section @ref QVNumericPlotWidget.

@see QVCPUPlot
@see QVHistogramPlot

@ingroup qvgui
*/

class QVNumericPlot : public QVPlot
	{
	Q_OBJECT
	public:
		/// @brief Constructs a QVNumericPlot with a given string identifier, and iteration's parameters.
		/// Construct a new plot that shows int and double properties graphs. This plot shows decorations, it hasn't got a pie graph, don't brush the data
		/// and autoshow. It can show graphs of int and double linked properties of different workers.
		///
		/// @param name Identifier for the object to be created.
		/// @param time True if the iterations are based on clock time (in second hundredths), and false if they are based on worker's iterations.
		/// @param step Set the base measure to plot's iterations ratio.
		/// @param parent Identifier for the parent widget.
		QVNumericPlot(const QString name = QString (), bool time = true, int step = 10, QWidget *parent = 0);

#ifndef DOXYGEN_IGNORE_THIS
	protected:
		/// @brief Links an unspecified input property.
		///
		/// This method safely links only int or double properties from workers in a thread safe manner.
		/// When the user creates a link with an unispecified output property, the source container calls this destiny's method.
		/// If the destiny container is a QVNumericPlot lets only links from int or double properties, creates a property with the
		/// source property's name and calls the source container in order to it do the link with this new property.
		///
		/// See also linkProperty(QVWorker &worker, const QString propertyName).
		/// @param sourceContainer The container that whants to be linked.
		/// @param sourcePropName The name of the source property to be linked.
		/// @param linkType The link's type.
		bool linkUnspecifiedInputProperty(QVPropertyContainer *sourceContainer, QString sourcePropName, LinkType linkType = AsynchronousLink);

	private:
		/// @brief Returns a list with a int or double property curve's names.
		/// Return a list with the name of the given property, that only contains one curve.
		///
		/// @param property The property name.
		QStringList getPropertyCurvNames(QString property) const;

		/// @brief Returns a list with a int or double property curve's value.
		/// Return a list with the value of the given property, that only contains one curve.
		///
		/// @param property The property name.
		QList<double> getPropertyCurvValues(QString property) const;

		/// @brief Returns a list with a int or double property curve's index.
		/// Return a list with the value 1 for the given property order, that only contains one curve, the first.
		///
		/// @param property The property name.
		QList<int> getPropertyCurvOrders(QString property) const;
	};
#endif
#endif

